@@ -18,20 +18,16 @@ def remove_dead_locks(db_name, ignore=None):
 			continue
 		lock_parts = lock.split(".")
 		time_ns = lock_parts[-3]
-		if time.time_ns() - int(time_ns) > LOCK_TIMEOUT * 1_000_000_000:
+		if time.monotonic_ns() - int(time_ns) > LOCK_TIMEOUT * 1_000_000_000:
 			Path(lock).unlink()
 			print(f"Found dead lock ({lock}). Remove")
 
 
 def path_str(db_name, lock_id, time_ns, lock_type):
-	path = f"{config.storage_directory}/"
-	if "/" in db_name:
-		db_name = db_name.split("/")
-		db_name[-1] = f".{db_name[-1]}"
-		db_name = "/".join(db_name)
-	else:
-		path += "."
-	return f"{path}{db_name}.{lock_id}.{time_ns}.{lock_type}.lock"
+	path = f"{config.storage_directory}/.ddb/"
+	path = f"{path}{db_name}.{lock_id}.{time_ns}.{lock_type}.lock"
+	Path(path).parent.mkdir(parents=True, exist_ok=True)
+	return path
 
 
 def check_if_lock_exists(db_name: str, thread_id: str, lock_type: str):
@@ -44,11 +40,14 @@ def find_locks(lock_type: str, db_name: str):
 
 
 def is_oldest_need_lock(lock_id, db_name):
-	lock_candidates = find_locks("need*", db_name)
-	lock_candidates = [x.split(".")[:-2][-2:] for x in lock_candidates]
-	oldest_candidate = min(lock_candidates, key=lambda x: int(x[1]))[0]
-	return oldest_candidate == lock_id
-
+	need_locks = find_locks("need*", db_name)
+	if len(need_locks) == 0:
+		return True
+	need_locks = [x.split(".")[:-2][-2:] for x in need_locks]  # Get the lock_id and time_ns
+	need_locks = sorted(need_locks, key=lambda x: int(x[0]))  # Sort by lock_id
+	need_locks = sorted(need_locks, key=lambda x: int(x[1]))  # Sort by time_ns
+	oldest_need_lock = need_locks[0][0]
+	return oldest_need_lock == lock_id
 
 
 class AbstractLock(object):
@@ -67,7 +66,7 @@ class AbstractLock(object):
 			and the current time in nanoseconds as the time.
 		"""
 		self.id = str(threading.get_native_id())
-		self.time_ns = time.time_ns()
+		self.time_ns = time.monotonic_ns()
 		self.db_name = db_name
 		self.path = None
 
